@@ -1,7 +1,7 @@
 /*
 
   FITSPNG     FITS to PNG converter
-  Copyright (C) 2006-x  Filip Hroch, Masaryk University, Brno, CZ
+  Copyright (C) 2006-2017  Filip Hroch, Masaryk University, Brno, CZ
 
   Fitspng is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 #define SKIP 10
 
 
-
 int icut(float flux, int low, int high)
 {
   int iflux;
@@ -77,7 +76,6 @@ void XYZ_Luv(float X, float Y, float Z, float Yn, float xw, float yw,
 
   yy = Y/Yn;
   if( yy > 0.0088565f /* = powf(6.0/29.0,3)*/ )
-    /*    *L = 116.0f*powf(yy,0.3333333f) - 16.0f;*/
     *L = 116.0f*cbrtf(yy) - 16.0f;
   else
     *L = 903.30f*yy;   /* = pow(26.0/3.0,3) */
@@ -118,7 +116,7 @@ void Luv_XYZ(float L, float u, float v, float Yn, float xw, float yw,
 
 void XYZ_sRGB(float X, float Y, float Z, float *R, float *G, float *B)
 {
-  /* correction to D65 color temperature */
+  /* correction to D65 colour temperature */
   X = X*0.950456f;
   Z = Z*1.088754f;
 
@@ -132,7 +130,7 @@ void XYZ_AdobeRGB(float X, float Y, float Z, float *R, float *G,float *B)
 {
   const float q = 1.0f/2.19921875f;
 
-  /* correction to D65 color temperature */
+  /* correction to D65 colour temperature */
   X = X*0.950456f;
   Z = Z*1.088754f;
 
@@ -267,7 +265,6 @@ int fitspng(char *fitsname, char *png, int bit_depth,
   int naxis,bitpix,status,nullval=0,fpixel=1;
   int i,j,k,m,n,lim;
   char line[FLEN_CARD];
-  char *fw;
 
   /* data scaling */
   float r,thresh, slope;
@@ -281,7 +278,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
   png_text text_ptr[NSTDKEYS];
   char buf[NFKEYS*FLEN_CARD + 100];
   png_bytep *row_pointers;
-  char *tm[6];
+  char *tm[6], *c, *c0;
 
   /* intensity scaling */
   int ii,jj;
@@ -289,7 +286,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
   int iflux;
   float *pic;
 
-  /* display colorspace */
+  /* display colour-space */
   void (*XYZ_RGB)(float,float,float,float *,float *,float *);
 
   /* --------------------------------------------------------------------*/
@@ -313,6 +310,9 @@ int fitspng(char *fitsname, char *png, int bit_depth,
   for( i = 0; i < NSTDKEYS; i++)
     stdvalues[i] = NULL;
 
+  for( i = 0; i < 6; i++ )
+    tm[i] = NULL;
+
   if( verb )
     fprintf(stderr,"Finishing initialisation..\n");
 
@@ -320,7 +320,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
   /* Part: Load input FITS */
 
   /* check whatever filename has extension - usefull only for selecting
-     of single bands in color images
+     of single bands in colour images
   */
 
   fits_open_image(&f,fitsname, READONLY, &status);
@@ -334,14 +334,14 @@ int fitspng(char *fitsname, char *png, int bit_depth,
     goto finish;
 
   if( !(naxis == 2 || naxis == 3)) {
-    fprintf(stderr,"Crash: Only gray or color FITS files are supported.\n");
+    fprintf(stderr,"Crash: Only grey or colour FITS files are supported.\n");
     goto finish;
   }
 
   if( naxis == 3 ) {
     fits_read_key(f,TSTRING,"CSPACE",line,NULL,&status);
     if( status || strstr(line,"XYZ") == NULL ) {
-      fprintf(stderr,"Crash: Only CIE 1931 XYZ colorspace is supported yet.\n");
+      fprintf(stderr,"Crash: Only CIE 1931 XYZ colour-space is supported yet.\n");
       goto finish;
     }
   }
@@ -395,14 +395,14 @@ int fitspng(char *fitsname, char *png, int bit_depth,
     n = (naxes[0]*naxes[1])/(SKIP*SKIP);
     if( (d = malloc((n+1)*sizeof(float))) ) {
 
-      /* scale by Y component for color images */
+      /* scale by Y component for colour images */
       k = naxis == 2 ? 0 : n;
 
       for( i = 0, j = 0; j < n; i = i + SKIP, j++)
 	d[j] = pic[k+i];
       med = qmed(n,d,n/2+1);
 
-      /* only greater than median part of histogram is used */
+      /* only median+ (right tail) part of histogram is examined */
       int imax = naxes[0]*naxes[1] - SKIP - 1;
       for( i = 0, j = 0; j < n && i < imax; i = i + SKIP) {
 	r = pic[k+i] - med;
@@ -423,7 +423,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
       fprintf(stderr,"There is no room for a scaling buffer.\n");
 
     if( verb )
-      fprintf(stderr,"Finishing estimate of scale parameters..\n");
+      fprintf(stderr,"Finishing of estimate of scale parameters..\n");
   }
 
 
@@ -495,7 +495,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
       /* 8-bit */
       else if( bit_depth == 8 ) {
 
-	/* grayscale */
+	/* greyscale */
         if( naxis == 2 ) {
 
 	  flux = farray2d(pic,naxes[0],naxes[1],ii,jj,scale);
@@ -506,7 +506,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
 	}
 	else if( naxis == 3 ) {
 
-	  /* color */
+	  /* colour */
 	  Z = farray2d(pic,naxes[0],naxes[1],ii,jj,scale);
 	  Y = farray2d(pic+naxes[0]*naxes[1],naxes[0],naxes[1],ii,jj,scale);
 	  X = farray2d(pic+2*naxes[0]*naxes[1],naxes[0],naxes[1],ii,jj,scale);
@@ -520,7 +520,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
 	  /* apply tone curve */
 	  L = 100.0f*fscale(L/100.0f,ctype,0.0,1.0,f0,zero);
 
-	  /* color saturation */
+	  /* colour saturation */
 	  float hue = atan2f(v,u);
 	  float chroma = satur*sqrtf(v*v + u*u);
 	  u = chroma*cosf(hue);
@@ -539,7 +539,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
 	    Z = w*Z + w1*s;
 	  }
 
-	  /* convert to output colorspace */
+	  /* convert to output colour-space */
 	  (*XYZ_RGB)(X,Y,Z,&R,&G,&B);
 
 	  image[m] = icut(255.0f*R,0,lim);
@@ -600,7 +600,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
     fp = stdout;
 
   if (!fp) {
-    fprintf(stderr,"Crash: File output initialisation failed.\n");
+    fprintf(stderr,"Crash: Initialising of an output file failed.\n");
     goto finish;
   }
 
@@ -679,19 +679,35 @@ int fitspng(char *fitsname, char *png, int bit_depth,
   stdvalues[2] = strdup(buf);
   stdvalues[3] = strdup("");
 
-  /* decode right time (round fractional seconds and adds timezone) */
-  for( i = 0, (fw = strtok(fval[3],"-T: ")); fw; fw=strtok(NULL,"-T: "), i++ ){
-    if( i < 6 )
-      tm[i] = fw;
+  /* decode time (round fractional seconds and adds timezone),
+     Standard FITS headers must contains DATE-OBS as
+     YYYY-MM-DDTHH:MM:SSS.SSS, however the rule is violated
+     by many software writing obsolete YYYY-MM-DD keyword.
+  */
+  for( i = 0, c = fval[3], c0 = fval[3]; *c != '\0'; c++) {
+    if( *c == '-' || *c == 'T' || *c == ':' || *c == ' ' || *(c+1) == '\0') {
+      n = c - c0;
+      if( *(c+1) == '\0' )
+	n++;
+      tm[i++] = strndup(c0,n);
+      c0 = c + 1;
+    }
   }
-  if( i >= 6 ) {
+  /* This loop is my own bugy implementation of bugy strtok
+     (https://sourceware.org/bugzilla/show_bug.cgi?id=16640) */
+  if( i == 6 ) {
     if( sscanf(tm[5],"%f",&r) == 1 )
       i = rint(r);
     else
       i = 0;
     sprintf(buf,"%s-%s-%s %s:%s:%02d GMT",tm[0],tm[1],tm[2],tm[3],tm[4],i);
     stdvalues[4] = strdup(buf);
+  } else if( i == 3 ) {
+    sprintf(buf,"%s-%s-%s",tm[0],tm[1],tm[2]);
+    stdvalues[4] = strdup(buf);
   }
+  else
+    stdvalues[4] = strdup("");
 
   stdvalues[5] = strdup("Created by FITSPNG.");
   stdvalues[6] = strdup("");
@@ -729,7 +745,7 @@ int fitspng(char *fitsname, char *png, int bit_depth,
 #endif
 
   if( (row_pointers = malloc(height*sizeof(row_pointers))) == NULL ) {
-    fprintf(stderr,"There is no room for rows of image.\n");
+    fprintf(stderr,"There is no room for all rows of image.\n");
     png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
     goto finish;
   }
@@ -755,6 +771,8 @@ int fitspng(char *fitsname, char *png, int bit_depth,
     free(fval[i]);
   for( i = 0; i < NSTDKEYS; i++)
     free(stdvalues[i]);
+  for( i = 0; i < 6; i++ )
+    free(tm[i]);
 
   free(image);
   free(pic);
